unit UAnalogIN_ExternTriggerStart;

interface

implementation

uses
  SysUtils,
  ConsoleHelper,
  MAI_v2_DLL;


var
	CardID, BlockSize, SizeOfLastBlockRead,  ValuesPerChannel , NumberOfADChannels, CurrentNumberOfValues,MaxValuesToGet,ValuesInFifo: Cardinal;
  i,Result                : Integer;
  KeyThatWasPressed       : AnsiChar;
  StreamingMode           : TStreamingMode;
  RawData		              : array of Integer;
	ScaledData              : array of Double;
  ADAnzeige               : array of Double;

begin
//////////////////////////////////////////////////////////////////////////////////////////////
//  Parameter der Messung	                                                       						//
//////////////////////////////////////////////////////////////////////////////////////////////
  NumberOfADChannels		:= 4;
  StreamingMode			    := smOFF;
  ValuesPerChannel		  := 2000;
  MaxValuesToGet        := 1000000;
//////////////////////////////////////////////////////////////////////////////////////////////
//  CardID der ersten, verfgbaren Karte verwenden	                             						//
//////////////////////////////////////////////////////////////////////////////////////////////
  CardID := GetFirstDevice();
	if (CardID < 0) then
	begin
		PrintError(CardID, CardID);
	 	Exit;
	end;

  WriteLn('Verwende Karte mit ID 0.');
  Writeln('Serial Number : '+GetSerialNumber(CardID));            // Seriennummer der Karte


//////////////////////////////////////////////////////////////////////////////////////////////
// Kanle der Kanalliste hinzufgen									                             						//
//////////////////////////////////////////////////////////////////////////////////////////////
	for i := 0 to NumberOfADChannels-1 do
	begin
		Result := AD_AddChannelToList(	CardID,												// ID der anzusprechenden Karte
                                    i,												  	// Kanalnummer
                                    oversamplingDISABLED,					// Funktion um Strungen des Messsignals zu reduzieren
                                    gainDisabled,									// Verstrkungsfaktor
                                    polBIPOLAR  );								// ein Wechsel der Polatitt wird vom Basic-Karten nicht untersttzt

		if (Result <> 0) then 																	      // Fehler prfen
		begin
			PrintError(CardID, Result);
      Exit;
		end; 
	end;

//////////////////////////////////////////////////////////////////////////////////////////////
// Frequenz setzen																			                                    //
//////////////////////////////////////////////////////////////////////////////////////////////
	Result := AD_SetFrequency(	CardID,
								              50000   );											    	// Abtastrate pro Kanal
	if (Result <> 0) then 																		        // Fehler prfen
	begin
	  PrintError(CardID, CardID);
	 	Exit;
	end;

//////////////////////////////////////////////////////////////////////////////////////////////
// Parametrieren der Karte fr eine(n) externe(n) Triggerung / Takt							//
//////////////////////////////////////////////////////////////////////////////////////////////
	Result := ConfigCard(CardID, trigEXTERN_TRIGGER, tedgeFALLING_EDGE, False, 0);
	if (Result <> 0) then 																		        // Fehler prfen
	begin
	  PrintError(CardID, CardID);
	 	Exit;
	end;
//////////////////////////////////////////////////////////////////////////////////////////////
// Einstellung fr sptere Messwerterfassung												                        //
//////////////////////////////////////////////////////////////////////////////////////////////
	if (StreamingMode = smON) then 															// AD-Werte aller Kanle in ein gemeinsames Fifo
	begin 																						          // User muss sich um das zerreien der Daten kmmern
		Result := SetStreamingMode(CardID, StreamingMode  );			// StreamingMode aktivieren (Standard: aus)
		if (Result <> 0) then
		begin 
      PrintError(CardID, CardID);
	 	  Exit;
		end;
		BlockSize := ValuesPerChannel * NumberOfADChannels;
	end
	else
		BlockSize := ValuesPerChannel;														// Jeder AD-Kanal hat eigenes Fifo, zerreien nicht mehr ntig



//////////////////////////////////////////////////////////////////////////////////////////////
// Parametrieren der Karte und Verarbeitung aller zuvor gettigten Eingaben / Einstellungen //
//////////////////////////////////////////////////////////////////////////////////////////////
	Result := ConfigMeasure(CardID);
	if (Result <> 0) then
	begin
      PrintError(CardID, CardID);
	 	  Exit;
	end;


//////////////////////////////////////////////////////////////////////////////////////////////
// Messung anstarten																		                                    //
//////////////////////////////////////////////////////////////////////////////////////////////
	Result := StartMeasure(CardID);
	if (Result <> 0) then
	begin
     PrintError(CardID, CardID);
     Exit;
	end;


	SetLength(RawData,   BlockSize);
	SetLength(ScaledData,BlockSize);
	SetLength(ADAnzeige, NumberOfADChannels);
  Writeln('Mesung gestartet. Warte auf externes Trggerereignis ( <ESC> =  Abbruch) ...');
  Writeln;

//////////////////////////////////////////////////////////////////////////////////////////////
// Messdaten abholen																		                                    //
//////////////////////////////////////////////////////////////////////////////////////////////

  CurrentNumberOfValues := 0;
	while (CurrentNumberOfValues < MaxValuesToGet) do 											// Bedingung fr Beendigung der Schleife
	begin
    if (KeyPressed(KeyThatWasPressed)) then
    begin
        if(KeyThatWasPressed = #27) then
        begin
           WriteLn('Sie haben <ESC> gedrueckt.');
				   break;
        end;

    end;
    Result := AD_GetNumberOfValues(CardID, ValuesInFifo);										// gibt Anzahl der in den Fifos befindlichen Werte zurck
		if (Result <> 0) then 
		begin
      PrintError(CardID, Result);
      break;
		end;

		if (ValuesInFifo >= BlockSize) then 														// wenn gengend Werte vorliegen
		begin 
			if (StreamingMode = smOFF) then 															// wenn jeder AD-Kanal eigenes Fifo verwendet
			begin 
				for i := 0 to NumberOfADChannels-1 do									// jeder ADKanal wird durchlaufen
				begin
          SizeOfLastBlockRead :=  BlockSize;

					Result := AD_ReadData(	CardID,											 // ID der verwendeten Karte
                                  i,												   // Nummer des Kanallisteneintrages (Grund: Wird ein Kanal mehrmals in die Kanalliste aufgenommen, muss zur Unterscheidung die Nummer des Kanallisteneintrages angegeben werden.)
                                  @RawData[0],								 // Rohdaten
                                  @ScaledData[0],							 // Skalierte Daten
                                  SizeOfLastBlockRead,				 // Anzahl der Werte
                                  partONLYFULLBLOCKS,						 // Ganze Blcke / Teilblcke	(0 - ganz , 1 - teil)
                                  skipGETCAPTUREDVALUES    );		 // Erfasste Werte verwerfen		(0 - false, 1 - true)
					if (Result <> 0) then
					begin
              PrintError(CardID, Result);
              break;
					end;

					ADAnzeige[i] := ScaledData[0];											// der jeweilig erste Werte wird zur Ausgabe auf der Konsole bereitgestellt
					CurrentNumberOfValues := CurrentNumberOfValues + SizeOfLastBlockRead;				// Zhler fr die Anzahl der erfassten Werte
				end;
			end 

			else 																			// alle AD-Werte in einem Fifo
			begin

        SizeOfLastBlockRead :=  BlockSize;

				Result := AD_ReadData(	CardID,											// ID der verwendeten Karte
                                0,													// 0, weil alles in einem Fifo
                                @RawData[0],							  // Rohdaten
                                @ScaledData[0],							 // Skalierte Daten
                                SizeOfLastBlockRead,				 // Anzahl der Werte
                                partONLYFULLBLOCKS,						 // Ganze Blcke / Teilblcke	(0 - ganz , 1 - teil)
                                skipGETCAPTUREDVALUES    );		 // Erfasste Werte verwerfen		(0 - false, 1 - true)
				if (Result <> 0) then
				begin
			   PrintError(CardID, Result);
         break;
				end;

				for i := 0 to  NumberOfADChannels-1 do								// Werte liegen im gelesenen Burst vor: Kanal0, Kanal1, ..., Kanal0, Kanal1
					ADAnzeige[i] := ScaledData[i];

				CurrentNumberOfValues := CurrentNumberOfValues + SizeOfLastBlockRead;					// Zhler fr die Anzahl der erfassten Werte
			end;

      Write(Char(13)+'  ' + InttoStr( currentNumberOfValues )+ '/' + InttoStr( MaxValuesToGet )+'     Spannungen: ');
      for i := 0 to  NumberOfADChannels-1 do
      begin
          Write(FloatToStrF( ADAnzeige[i],ffFixed, 4, 2) );
          Write('    ');
      end;
      Sleep(1);
		end;

	end;

  //////////////////////////////////////////////////////////////////////////////////////////////
  // Messung anhalten																			                                    //
  //////////////////////////////////////////////////////////////////////////////////////////////
	Result := StopMeasure(CardID);
	if (Result <> 0) then
	begin
    PrintError(CardID, CardID);
    Exit;
	end;

  WriteLn;
  WriteLn;
  Writeln('Fertig. (Bitte <Enter> druecken...)');
  Read(KeyThatWasPressed);
end.
