unit UAnalogIN_TTL;

interface

implementation

uses
  SysUtils,
  ConsoleHelper,
  MAI_v2_DLL;


var
	CardID,  SizeOfLastBlockRead,  ValuesPerChannel , NumberOfADChannels,NumberOfTTLChannels, CurrentNumberOfValues,MaxValuesToGet,ValuesInFifo: Cardinal;
  i,Result                : Integer;
  KeyThatWasPressed       : AnsiChar;
  StreamingMode           : TStreamingMode;
  RawData		              : array of Integer;
  TTLData                 : array of Cardinal;
	ScaledData              : array of Double;
  ADAnzeige               : array of Double;

begin
//////////////////////////////////////////////////////////////////////////////////////////////
//  Parameter der Messung	                                                       						//
//////////////////////////////////////////////////////////////////////////////////////////////

  NumberOfADChannels		:= 2;
  NumberOfTTLChannels   := 1;
  ValuesPerChannel		  := 100;
  MaxValuesToGet        := 10000;

	SetLength(RawData,    ValuesPerChannel);
	SetLength(ScaledData ,ValuesPerChannel);
	SetLength(TTLData,ValuesPerChannel);

	SetLength(ADAnzeige, NumberOfADChannels);

//////////////////////////////////////////////////////////////////////////////////////////////
//  CardID der ersten, verfgbaren Karte verwenden	                             						//
//////////////////////////////////////////////////////////////////////////////////////////////
  CardID := GetFirstDevice();
	if (CardID < 0) then
	begin
		PrintError(CardID, CardID);
	 	Exit;
	end;

  WriteLn('Verwende Karte mit ID 0.');
  Writeln('Serial Number : '+GetSerialNumber(CardID));            // Seriennummer der Karte

  Writeln;
  Result := ClearAllChannelLists(CardID) ;
  if (Result <> 0) then
  begin
			PrintError(CardID, Result);
      Exit;
	end;

//////////////////////////////////////////////////////////////////////////////////////////////
// Kanle der Kanalliste hinzufgen									                             						//
//////////////////////////////////////////////////////////////////////////////////////////////
	for i := 0 to NumberOfADChannels-1 do
	begin
		Result := AD_AddChannelToList(	CardID,												// ID der anzusprechenden Karte
                                    i,												  	// Kanalnummer
                                    oversamplingDISABLED,		 			// Funktion um Strungen des Messsignals zu reduzieren
                                    gainDisabled,									// Verstrkungsfaktor
                                    polBIPOLAR  );								// ein Wechsel der Polatitt wird vom Basic-Karten nicht untersttzt

		if (Result <> 0) then 																	      // Fehler prfen
		begin
			PrintError(CardID, Result);
      Exit;
		end;
	end;

	for i := 0 to NumberOfTTLChannels-1 do
	begin
    Result := TTL_AddChannelToList(	CardID,											// ID der anzusprechenden Karte
										                i	,												  // Kanalnummer
                                    True,                       // Synchron zur AD Kanalliste
                                    dptBit);
    if (Result <> 0) then
    begin
      PrintError(CardID, CardID);
      Exit;
    end;

	end;

//////////////////////////////////////////////////////////////////////////////////////////////
// Frequenzen setzen																			                                    //
//////////////////////////////////////////////////////////////////////////////////////////////
	Result := AD_SetFrequency(	CardID,
								              1000   );											    	// Abtastrate pro Kanal
	if (Result <> 0) then 																		        // Fehler prfen
	begin
	  PrintError(CardID, CardID);
	 	Exit;
	end;



//////////////////////////////////////////////////////////////////////////////////////////////
// Parametrieren der Karte und Verarbeitung aller zuvor gettigten Eingaben / Einstellungen //
//////////////////////////////////////////////////////////////////////////////////////////////
	Result := ConfigMeasure(CardID);
	if (Result <> 0) then
	begin
      PrintError(CardID, CardID);
	 	  Exit;
	end;

//////////////////////////////////////////////////////////////////////////////////////////////
// Messung starten																		                                    //
//////////////////////////////////////////////////////////////////////////////////////////////
	Result := StartMeasure(CardID);
	if (Result <> 0) then
	begin
     PrintError(CardID, CardID);
     Exit;
	end;




//////////////////////////////////////////////////////////////////////////////////////////////
// Messdaten abholen																		                                    //
//////////////////////////////////////////////////////////////////////////////////////////////

  CurrentNumberOfValues := 0;

	while (CurrentNumberOfValues < MaxValuesToGet) do 											// Bedingung fr Beendigung der Schleife
	begin
    if (KeyPressed(KeyThatWasPressed)) then
    begin
        if(KeyThatWasPressed = #27) then
        begin
           WriteLn('Sie haben <ESC> gedrueckt.');
				   break;
        end;

    end;
    Result := AD_GetNumberOfValues(CardID, ValuesInFifo);										// gibt Anzahl der in den Fifos befindlichen Werte zurck
		if (Result <> 0) then
		begin
      PrintError(CardID, Result);
      break;
		end;
		if (ValuesInFifo >= ValuesPerChannel) then 														// wenn gengend Werte vorliegen
		begin

      SizeOfLastBlockRead :=  ValuesPerChannel;
        /////////////////////////////////////////////////////////////////////////
        // Analogdaten abholen										                             //
        /////////////////////////////////////////////////////////////////////////

				for i := 0 to NumberOfADChannels-1 do									// jeder ADKanal wird durchlaufen
				begin

					Result := AD_ReadData(	CardID,											 // ID der verwendeten Karte
                                  i,												   // Nummer des Kanallisteneintrages (Grund: Wird ein Kanal mehrmals in die Kanalliste aufgenommen, muss zur Unterscheidung die Nummer des Kanallisteneintrages angegeben werden.)
                                  @RawData[0],								 // Rohdaten
                                  @ScaledData[0],							 // Skalierte Daten
                                  SizeOfLastBlockRead,				 // Anzahl der Werte
                                  partONLYFULLBLOCKS,						 // Ganze Blcke / Teilblcke	(0 - ganz , 1 - teil)
                                  skipGETCAPTUREDVALUES    );		 // Erfasste Werte verwerfen		(0 - false, 1 - true)
					if (Result <> 0) then
					begin
              PrintError(CardID, Result);
              break;
					end;

					ADAnzeige[i] := ScaledData[0];											// der jeweilig erste Werte wird zur Ausgabe auf der Konsole bereitgestellt
				end;

        /////////////////////////////////////////////////////////////////////////
        // Digitaldaten abholen										                             //
        /////////////////////////////////////////////////////////////////////////

		    Result := TTL_ReadData(	CardID,														// ID der Karte
                                @TTLData[0],											// Daten der TTL'ler
                                ValuesInFifo,							// Blockgre, nach Ausfhrung: Anzahl tatschlich gelesener Werte
                                partONLYFULLBLOCKS,									//
                                skipGETCAPTUREDVALUES );
        if (Result <> 0) then
        begin
            PrintError(CardID, Result);
            break;
        end;

      CurrentNumberOfValues := CurrentNumberOfValues + SizeOfLastBlockRead;				// Zhler fr die Anzahl der erfassten Werte

		end;




     Write(Char(13)+'  ' + InttoStr( currentNumberOfValues )+' / ' + InttoStr( MaxValuesToGet )+',  Volt: ');
      for i := 0 to  NumberOfADChannels-1 do
      begin
          Write(FloatToStrF( ADAnzeige[i],ffFixed, 4, 2) );
          Write(' ');
      end;
      Write(', TTL: ' + InttoStr( TTLData[0])+'            ');

      Sleep(1);
	end;

  //////////////////////////////////////////////////////////////////////////////////////////////
  // Messung anhalten																			                                    //
  //////////////////////////////////////////////////////////////////////////////////////////////
	Result := StopMeasure(CardID);
	if (Result <> 0) then
	begin
    PrintError(CardID, CardID);
    Exit;
	end;

  WriteLn;
  WriteLn;
  Writeln('Fertig. (Bitte <Enter> druecken...)');
  Read(KeyThatWasPressed);
end.

