unit UDigitalFilter;

interface

implementation

uses
  SysUtils,
  ConsoleHelper,
  MAI_v2_DLL;


var
	CardID,  SizeOfLastBlockRead,  ValuesPerChannel, CurrentNumberOfValues,MaxValuesToGet,ValuesInFifo,nCardsSWExtensions, DummyIndex: Cardinal;
  i,Result                : Integer;
  KeyThatWasPressed       : AnsiChar;
  RawData		              : array of Integer;
	ScaledData              : array of Double;
  IIRnotFIR               : Boolean;
  SampleRate              : Double;

  nFilterType             : TFilterType;
  dLeftCutoffFreq         : Double;
  dRightCutoffFreq        : Double;
  dPassapsorbtion         : Double;
  dStopBandAttenuation    : Double;
  nIIRDesignMethod        : TIIRFilterDesign;
  nFilterOrder            : Cardinal;
  // Output Parameters for Design Methods:
  nFilterMode             : Cardinal;
  FilterCoefficients      : array of Double;

begin
//////////////////////////////////////////////////////////////////////////////////////////////
//  Parameter der Messung	                                                       						//
//////////////////////////////////////////////////////////////////////////////////////////////
  ValuesPerChannel		  := 200;
  MaxValuesToGet        := 100000;
  IIRnotFIR             := True;
  SampleRate            := 5000;
  nFilterType           := ftLOWPASS;
  nFilterOrder			    := 10;
  nIIRDesignMethod      := iirBUTTERWORTH;
  dLeftCutoffFreq       := 23.3;
  dRightCutoffFreq      := 400.1;
  dPassapsorbtion       :=  0.9;
  dStopBandAttenuation  :=  0.1;
	SetLength(FilterCoefficients,4096);
//////////////////////////////////////////////////////////////////////////////////////////////
// zuerst einen neuen Filter entwerfen														//
//////////////////////////////////////////////////////////////////////////////////////////////
	if(IIRnotFIR)then// design IIR Filter
	begin
    Writeln('designing an IIR Filter ... (see sourcecode)');
  	Result := Realtime_DesignIIRFilter(SampleRate,  // use AD Samplerate
										  nFilterType,
										  dLeftCutoffFreq,
										  dRightCutoffFreq,
										  false, // Frequencies are NOT normalized
										  dPassapsorbtion,
										  dStopBandAttenuation,
										  nIIRDesignMethod,
										  nFilterOrder,
										  // Output Parameters:
										  nFilterMode,
										  @FilterCoefficients[0]);

	end
	else      // design FIR Filter
	begin
  	 Writeln('designing an FIR Filter ... (see sourcecode)');
     Result := Realtime_DesignFIRFilter(SampleRate,  // use AD Samplerate
										  nFilterType,
										  dLeftCutoffFreq,
										  dRightCutoffFreq,
										  false, // Frequencies are NOT normalized
										  nFilterOrder,
										  firwinRectangular,
										  0,
										  false, // try Scale Passband to 1
										  // Output Parameters:
										  nFilterMode,
										  @FilterCoefficients[0]);

	end;
  if (Result <> 0) then
  begin
			PrintError(CardID, Result);
      Exit;
	end;
//////////////////////////////////////////////////////////////////////////////////////////////
//  CardID der ersten verfgbaren Karte verwenden	                              						//
//////////////////////////////////////////////////////////////////////////////////////////////
  CardID := GetFirstDevice();
	if (CardID < 0) then
	begin
		PrintError(CardID, CardID);
	 	Exit;
	end;
//////////////////////////////////////////////////////////////////////////////////////////////
// Prfen, ob Filter SW Extension aktiviert ist   										                          //
//////////////////////////////////////////////////////////////////////////////////////////////
	nCardsSWExtensions := 0;
	Result := GetSoftwareExtension(CardID,nCardsSWExtensions);
  if (Result <> 0) then
  begin
			PrintError(CardID, Result);
      Exit;
	end;
	if(((nCardsSWExtensions shr Cardinal(swxOnline_Filter))and 1)=0) then // Masking the Bit for desired Function
  begin
    WriteLn;
    Writeln('Diese Karte hat keine Digital Filter SW Extension installiert. (Bitte <Enter> druecken...)');
    Read(KeyThatWasPressed);
    Exit;
	end;

  WriteLn('Verwende Karte mit ID 0.');
  Writeln('Serial Number : '+GetSerialNumber(CardID));            // Seriennummer der Karte

  Result := ClearAllChannelLists(CardID) ;
  if (Result <> 0) then
  begin
			PrintError(CardID, Result);
      Exit;
	end;
//////////////////////////////////////////////////////////////////////////////////////////////
// AD Kanal der Kanalliste hinzufgen									                             						//
//////////////////////////////////////////////////////////////////////////////////////////////


		Result := AD_AddChannelToList(	CardID,												// ID der anzusprechenden Karte
                                    0,												  	// Kanalnummer
                                    oversamplingDISABLED,					// Funktion um Strungen des Messsignals zu reduzieren
                                    gainDisabled,									// Verstrkungsfaktor
                                    polBIPOLAR  );								// ein Wechsel der Polatitt wird vom Basic-Karten nicht untersttzt

		if (Result <> 0) then 																	      // Fehler prfen
		begin
			PrintError(CardID, Result);
      Exit;
		end;


//////////////////////////////////////////////////////////////////////////////////////////////
// Frequenz setzen																			                                    //
//////////////////////////////////////////////////////////////////////////////////////////////
	Result := AD_SetFrequency(	CardID,
								              SampleRate   );						 			    	// Abtastrate
	if (Result <> 0) then 																		        // Fehler prfen
	begin
	  PrintError(CardID, CardID);
	 	Exit;
	end;

//////////////////////////////////////////////////////////////////////////////////////////////
// Filter setzen																			                                    //
//////////////////////////////////////////////////////////////////////////////////////////////
 	Result := Realtime_AddFilterEntry(CardID,   // Card ID
										0,  // Number of AD Channel in Channellist
										nFilterMode,  // Size of FFT Spectrum
										nFilterOrder,  // Type of FFT Spectrum
										@FilterCoefficients[0],  // Gain Factor
										1,  // Output Gain.
										DummyIndex);  // Out : Handle to the generated On-Card Online-Function (Some Cards/Functions allow later Change)
	if (Result <> 0) then 																		        // Fehler prfen
	begin
	  PrintError(CardID, CardID);
	 	Exit;
	end;

//////////////////////////////////////////////////////////////////////////////////////////////
// Parametrieren der Karte und Verarbeitung aller zuvor gettigten Eingaben / Einstellungen //
//////////////////////////////////////////////////////////////////////////////////////////////
	Result := ConfigMeasure(CardID);
	if (Result <> 0) then
	begin
      PrintError(CardID, CardID);
	 	  Exit;
	end;


//////////////////////////////////////////////////////////////////////////////////////////////
// Messung anstarten																		                                    //
//////////////////////////////////////////////////////////////////////////////////////////////
	Result := StartMeasure(CardID);
	if (Result <> 0) then
	begin
     PrintError(CardID, CardID);
     Exit;
	end;


	SetLength(RawData,   ValuesPerChannel);
	SetLength(ScaledData,ValuesPerChannel);

//////////////////////////////////////////////////////////////////////////////////////////////
// Messdaten abholen																		                                    //
//////////////////////////////////////////////////////////////////////////////////////////////

  CurrentNumberOfValues := 0;
	while (CurrentNumberOfValues < MaxValuesToGet) do 											// Bedingung fr Beendigung der Schleife
	begin
    if (KeyPressed(KeyThatWasPressed)) then
    begin
        if(KeyThatWasPressed = #27) then
        begin
           WriteLn('Sie haben <ESC> gedrueckt.');
				   break;
        end;

    end;
    Result := AD_GetNumberOfValues(CardID, ValuesInFifo);										// gibt Anzahl der in den Fifos befindlichen Werte zurck
		if (Result <> 0) then 
		begin
      PrintError(CardID, Result);
      break;
		end;

		if (ValuesInFifo >= ValuesPerChannel) then 														// wenn gengend Werte vorliegen
		begin 

        SizeOfLastBlockRead :=  ValuesPerChannel;

				Result := AD_ReadData(	CardID,											// ID der verwendeten Karte
                                0,													// 0, weil alles in einem Fifo
                                @RawData[0],							  // Rohdaten
                                @ScaledData[0],							 // Skalierte Daten
                                SizeOfLastBlockRead,				 // Anzahl der Werte
                                partONLYFULLBLOCKS,						 // Ganze Blcke / Teilblcke	(0 - ganz , 1 - teil)
                                skipGETCAPTUREDVALUES    );		 // Erfasste Werte verwerfen		(0 - false, 1 - true)
				if (Result <> 0) then
				begin
			   PrintError(CardID, Result);
         break;
				end;


				CurrentNumberOfValues := CurrentNumberOfValues + SizeOfLastBlockRead;					// Zhler fr die Anzahl der erfassten Werte


      Write(Char(13)+'  ' + InttoStr( currentNumberOfValues )+ '/' + InttoStr( MaxValuesToGet )+'     Spannung: '+ FloattoStr(  ScaledData[0]));

      Sleep(1);
		end;

	end;

  //////////////////////////////////////////////////////////////////////////////////////////////
  // Messung anhalten																			                                    //
  //////////////////////////////////////////////////////////////////////////////////////////////
	Result := StopMeasure(CardID);
	if (Result <> 0) then
	begin
    PrintError(CardID, CardID);
    Exit;
	end;

  WriteLn;
  WriteLn;
  Writeln('Fertig. (Bitte <Enter> druecken...)');
  Read(KeyThatWasPressed);
end.
